///////////////////////////////////////////////////////////////////////
// FileName   :  "env.v"
// Author     :  Jordan (Jianping) Lu
// Department :  ECE of Concordia University
// Profile    :  Model checking environment of infifo16.v 
//               For any questions and errors in this code, please contact
//               me by email: jianping@ece.concordia.ca or phone: 
//               (514)848-8758. 
////////////////////////////////////////////////////////////////////////

`define  FWIDTH     16          // Width of the FIFO.
`define  FDEPTHIN   128         // Depth of the FIFO (contain 4 ATM cells).
`define  FCWIDTHIN  7           // Counter Width 
`define  DATWIDTH   16          // Width of the data cell
`define  PRTYWIDTH  2           // Width of the parity bits
`define  td         #2          // Register Q time
module environment (IFCLK);
input IFCLK;
wire                   IFCLK;         
wire [(`DATWIDTH-1):0] IDAT;           
wire [(`PRTYWIDTH-1):0]PRTY;          
wire                   ISOC;          
wire                   IAVALID_ICA4;   
wire  [1:0]            ICA32;          
wire                   IPOLL;          
wire                   ICA1;          
wire  [2:0]            IADDR_IWRENB;   
wire                   IWRENB1;
wire  [1:0]            IADDR43;
wire                   HECUDF;         
wire                   ICAINV;         
wire [2:0]             CELLPOST;      
wire [2:0]             CELLLEN;        
wire                   IBYTEPRTY;      
wire                   IPTYP;          
wire [1:0]             FIFODP;         
wire                   ICALEVEL0;      
wire                   IFRST;          
wire  [1:0]            IPRTY;          
wire                   PHYCELL;        
wire                   IFRDB;        
wire [(`DATWIDTH-1):0] IFDAT;         
wire                   IFRCA;
wire                   IFOCA;
wire                   WRITEB;
wire                   RD_PTR;        
wire                   WR_PTR;
IN_FIFO IN_FIFO0(  
                IFCLK,  
                IDAT,   
                ISOC,
                PRTY,   
                IAVALID_ICA4, 
                IADDR43,
                ICA32,
                ICA1,
                IADDR_IWRENB, 

                IWRENB1,
                IPOLL,
                HECUDF,
                ICAINV,
                CELLPOST,
                CELLLEN,
                IBYTEPRTY,  
                IPTYP,
                FIFODP,
                ICALEVEL0,
                IFRST,
                IFDAT,
                IFRDB,
                IFOCA,
                IFRCA,
                IPRTY,
                PHYCELL,
                FIFODATOUT,
                WRITEB,
                RD_PTR,
                WR_PTR
                );
iom  iom0 (  
                IFCLK,  
                IDAT,   
                ISOC,
                PRTY,   
                IAVALID_ICA4, 
                IADDR43,
                ICA32,
                ICA1,
                IADDR_IWRENB,
                IWRENB1,
                IPOLL,
                HECUDF,
                ICAINV,
                CELLPOST,
                CELLLEN,
                IBYTEPRTY,  
                IPTYP,
                FIFODP,
                ICALEVEL0,
                IFRST,
                IFDAT,
                IFRDB,
                IFOCA,
                IFRCA,
                IPRTY,
                PHYCELL,
                FIFODATOUT,
                WRITEB,
                RD_PTR,
                WR_PTR
                );
endmodule

// iom module
module iom (  
                IFCLK,  
                IDAT,   
                ISOC,
                PRTY,   
                IAVALID_ICA4, 
                IADDR43,
                ICA32,
                ICA1,
                IADDR_IWRENB,
                IWRENB1,
                IPOLL,
                HECUDF,
                ICAINV,
                CELLPOST,
                CELLLEN,
                IBYTEPRTY,  
                IPTYP,
                FIFODP,
                ICALEVEL0,
                IFRST,
                IFDAT,
                IFRDB,
                IFOCA,
                IFRCA,
                IPRTY,
                PHYCELL,
                FIFODATOUT,
                WRITEB,
                RD_PTR,
                WR_PTR
                );

input                   IFCLK;          
output [(`DATWIDTH-1):0] IDAT;          
output [(`PRTYWIDTH-1):0]PRTY;          
output                   ISOC;          
output                   IAVALID_ICA4;   
output  [1:0]            ICA32;          
output                   IPOLL;          
output                   ICA1;          
input [2:0]              IADDR_IWRENB;   
input                    IWRENB1;
input [1:0]              IADDR43;
output                   HECUDF;         
output                   ICAINV;         
output [2:0]             CELLPOST;       
output [2:0]             CELLLEN;        
output                   IBYTEPRTY;      
output                   IPTYP;          
input [1:0]              FIFODP;         
output                   ICALEVEL0;      
output                   IFRST;          
input [1:0]              IPRTY;          
input                    PHYCELL;       
output                   IFRDB;         
input                    IFRCA;         
input                    IFOCA;        
input                    WRITEB;       
input                    RD_PTR;        
input                    WR_PTR;
// Define outputs as registers
        
reg [(`DATWIDTH-1):0] IDAT;          
reg [(`PRTYWIDTH-1):0]PRTY;          
reg                   ISOC;          
reg                   IAVALID_ICA4;   
reg  [1:0]            ICA32;          
reg                   IPOLL;          
reg                   ICA1; 
reg                   HECUDF;         
reg                   ICAINV;         
reg [2:0]             CELLPOST;       
reg [2:0]             CELLLEN;        
reg                   IBYTEPRTY;      
reg                   IPTYP;            
reg                   ICALEVEL0;      
reg                   IFRST;        
reg                   IFRDB;
// Initialize the register variables
initial begin
          IDAT = 0;          
          PRTY = 0;          
          ISOC = 0;          
          IAVALID_ICA4 = 0;   
          ICA32 = 0;          
          IPOLL = 0;          
          ICA1 = 0; 
          HECUDF = 0;         
          ICAINV = 0;         
          CELLPOST = 0;       
          CELLLEN = 0;        
          IBYTEPRTY = 0;      
          IPTYP = 0;            
          ICALEVEL0 = 0;      
          IFRST = 0;        
          IFRDB = 0;
  end
// Define random variables
wire [3:0]             IDAT_ran4;
wire [(`PRTYWIDTH-1):0]PRTY_ran ;          
wire                   ISOC_ran ;          
wire                   IAVALID_ICA4_ran;   
wire  [1:0]            ICA32_ran;
wire                   IPOLL_ran;          
wire                   ICA1_ran;
wire                   HECUDF_ran;
wire                   ICAINV_ran;         
wire [2:0]             CELLPOST_ran;       
wire [2:0]             CELLLEN_ran;        
wire                   IBYTEPRTY_ran;      
wire                   IPTYP_ran;            
wire                   ICALEVEL0_ran;      
wire                   IFRST_ran;        
wire                   IFRDB_ran;
// Assign random value to random variables
assign   IDAT_ran4 = $ND(0, 1, 2, 3, 4, 5, 6, 7, 8,
                              9, 10, 11, 12, 13, 14, 15);
assign   PRTY_ran = $ND(0, 1, 2, 3) ;          
assign   ISOC_ran = $ND(0, 1);          
assign   IAVALID_ICA4_ran = $ND(0, 1);   
assign   ICA32_ran = $ND(0, 1, 2, 3);
assign   IPOLL_ran = $ND(0, 1);          
assign   ICA1_ran = $ND(0, 1);
assign   HECUDF_ran = $ND(0, 1);
assign   ICAINV_ran = $ND(0, 1);         
assign   CELLPOST_ran = $ND(0, 1, 2, 3, 4, 5, 6, 7);       
assign   CELLLEN_ran = $ND(0, 1, 2, 3, 4, 5, 6, 7);        
assign   IBYTEPRTY_ran = $ND(0, 1);      
assign   IPTYP_ran = $ND(0, 1);            
assign   ICALEVEL0_ran = $ND(0, 1);      
assign   IFRST_ran = $ND(0, 1);        
assign   IFRDB_ran = $ND(0, 1);
// Assign random variables to outputs
always @(posedge IFCLK) begin
          IDAT[3:0] = IDAT_ran4;          
          IDAT[7:4] = IDAT_ran4;          
          IDAT[11:8] = IDAT_ran4;          
          IDAT[15:12] = IDAT_ran4;          
          PRTY = PRTY_ran;          
          ISOC = ISOC_ran;          
          IAVALID_ICA4 = IAVALID_ICA4_ran;   
          ICA32 = ICA32_ran;          
          IPOLL = IPOLL_ran;          
          ICA1 = ICA1_ran; 
          HECUDF = HECUDF_ran;         
          ICAINV = ICAINV_ran;         
          CELLPOST = CELLPOST_ran;       
          CELLLEN = CELLLEN_ran;        
          IBYTEPRTY = IBYTEPRTY_ran;      
          IPTYP = IPTYP_ran;            
          ICALEVEL0 = ICALEVEL0_ran;      
          IFRST = IFRST_ran;        
          IFRDB = IFRDB_ran;
 end
endmodule
          
                  
 
